/*
 * Friends of friends for undirected graph.
 * 
 * Weights are not used.
 * 
 * Host is hard coded, be sure to change before running.
 */

import java.nio.ByteBuffer;
import java.nio.charset.Charset;
import java.nio.charset.CharsetDecoder;
import java.util.List;
import java.util.Scanner;
import java.util.TreeSet;

import org.apache.cassandra.thrift.*;
import org.apache.thrift.protocol.TProtocol;
import org.apache.thrift.transport.TFramedTransport;
import org.apache.thrift.transport.TTransport;
import org.apache.thrift.transport.TSocket;

public class FofF_W {
	public static void main(String[] args) throws Exception {
		
		String startNode = "";
		
		Scanner input = new Scanner(System.in);
		System.out.print("Starting Node #");
		startNode = input.next();
	
		// Set up decoder.
		Charset charset = Charset.forName("UTF-8");
		CharsetDecoder decoder = charset.newDecoder();
	
		// Set up connection.
		TTransport transport = new TFramedTransport(new TSocket("10.9.73.119", 9160));
		TProtocol protocol = new TBinaryProtocol(transport);
		Cassandra.Client client = new Cassandra.Client(protocol);
		transport.open();
		
		// Set the keyspace we are using.
		client.set_keyspace("CDM");

		// Set the parent of the column family
		ColumnParent parent = new ColumnParent();
		parent.column_family = "WEdges";

		// Set the consistency level
		ConsistencyLevel consistencyLevel = ConsistencyLevel.ONE;	
	
		// Set up the predicate for what we want out of the query
		SlicePredicate predicate = new SlicePredicate();
		predicate.addToColumn_names(ByteBuffer.wrap("start".getBytes()));
		predicate.addToColumn_names(ByteBuffer.wrap("end".getBytes()));
		
		// Set the expression for the index query
		IndexExpression indexExpression = new IndexExpression();
		indexExpression.column_name = ByteBuffer.wrap("start".getBytes("UTF-8"));
		indexExpression.setOp(IndexOperator.EQ);
		indexExpression.value = ByteBuffer.wrap(startNode.getBytes());
		
		// Create an IndexClause and add the index query to it.
		IndexClause indexClause = new IndexClause();
		indexClause.addToExpressions(indexExpression);
		indexClause.start_key = ByteBuffer.allocate(0);

		// Get the results.
		List<KeySlice> rangeResults = client.get_indexed_slices(parent, indexClause, predicate, consistencyLevel);

		// The friends set.
		TreeSet<String> friends = new TreeSet<String>();
		
		// Add them to the set
		for (KeySlice key : rangeResults) {
			for (ColumnOrSuperColumn col : key.columns) {
				String colName = decoder.decode(col.column.name).toString();
				String colValue = decoder.decode(col.column.value).toString();
				if (colName.contentEquals("end")) {
					friends.add(colValue);
				}
			}
		}
		
		// Get the friends going the other way
		indexExpression.column_name = ByteBuffer.wrap("end".getBytes("UTF-8"));
		
		rangeResults = client.get_indexed_slices(parent, indexClause, predicate, consistencyLevel);
		// Add them to the set
		for (KeySlice key : rangeResults) {
			for (ColumnOrSuperColumn col : key.columns) {
				String colName = decoder.decode(col.column.name).toString();
				String colValue = decoder.decode(col.column.value).toString();
				if (colName.contentEquals("start")) {
					friends.add(colValue);
				}
			}
		}
		
		// The friends of friends set.
		TreeSet<String> foff = new TreeSet<String>();
		
		// Get the friends of the friends.
		for (String friend : friends) {
			
			indexExpression.value = ByteBuffer.wrap(friend.getBytes());
			rangeResults = client.get_indexed_slices(parent, indexClause, predicate, consistencyLevel);
			
			// Add them to the set
			for (KeySlice key : rangeResults) {
				for (ColumnOrSuperColumn col : key.columns) {
					String colName = decoder.decode(col.column.name).toString();
					String colValue = decoder.decode(col.column.value).toString();
					if (colName.contentEquals("start")) {
						foff.add(colValue);
					}
				}
			}
		}
		
		// Get them going the other way
		indexExpression.column_name = ByteBuffer.wrap("start".getBytes());
		// Get the friends of the friends.
		for (String friend : friends) {
			
			indexExpression.value = ByteBuffer.wrap(friend.getBytes());
			rangeResults = client.get_indexed_slices(parent, indexClause, predicate, consistencyLevel);
			
			// Add them to the set
			for (KeySlice key : rangeResults) {
				for (ColumnOrSuperColumn col : key.columns) {
					String colName = decoder.decode(col.column.name).toString();
					String colValue = decoder.decode(col.column.value).toString();
					if (colName.contentEquals("end")) {
						foff.add(colValue);
					}
				}
			}
		}
		
		// Output the results
		ByteBuffer rowid = ByteBuffer.wrap(startNode.getBytes());
		ColumnPath path = new ColumnPath();
		path.column_family = "Nodes";
		path.column = ByteBuffer.wrap("name".getBytes());
		
		// Get the start node info
		ColumnOrSuperColumn result = client.get(rowid, path, consistencyLevel);
		System.out.println("Friends of friends for " + startNode + ":" + decoder.decode(result.column.value) + " are the following:");
		
		// Get the friends of friends info
		for (String f : foff) {
			rowid = ByteBuffer.wrap(f.getBytes());
			result = client.get(rowid, path, consistencyLevel);
			System.out.println("\t" + f + ":" + decoder.decode(result.column.value));
		}
		
		// Clean up.
		transport.flush();
		transport.close();
	}
}